% ---------------------------------------------------------------
% P=ReconstructUsingInverseMoments(a,b,P)
%
% Reconstruct and image I of size (a,b) from its
% moments up to order (m,n)
%
% This function is a numerically stable version of
%   ReconstructNaive.
% The numerical stability is achieved by using close formulas
% for the inverse of the Moments matrix.
%
% We only have this close formulas for the Moments matrix so,
% the other base matrixes (Fourier,...) can not be used here.
%
% Input parameters:
%       > a: rows for the reconstructed image
%       > b: columns for reconstructed image
%       > P: Projection coeficients obtained using moments.
%            See ProjectUsingMoments and Moments.
%            Size (m,n)
% Output parameters:
%       > I: Reconstruted Image
%            Matrix size (a,b)
%
% ---------------------------------------------------------------
function I=ReconstructUsingInverseMoments(a,b,P)
  [m,n]=size(P);

  % We use a direct formula to compute iBaa=inv(Baa)=inv(MomentsBasis(a,a))
  iBaa=InverseMomentsBasis(a);

  % We use a direct formula to compute iBbb=inv(Bbb)=inv(MomentsBasis(b,b))
  if (a~=b)
    iBbb=InverseMomentsBasis(b);
  else
    iBbb=iBaa;  % The square case is a common one
  end

  % We set the unknown moments to 0
  Pab=zeros(a,b);
  Pab(1:m,1:n)=P;

  I=iBaa' * Pab * iBbb;
