% ---------------------------------------------------------------
%
% Function performs test on numerical stability reported on the paper.
% In all outputs, the error is the average quadratic error per matrix element.
% Observe that the outputs are lower diagonal since the moment order (clumns) is 
% always below the image size (row).
%
% Input parameters:
%       > a: The maximum size for the images.
% Output parameters:
%       > Em: Error in the matrix inversion used in lemma 1
%               Em(i,j)=Error(inv(inv(M M')),M M')
%             with M a matrix moment of size i,j.
%       > Ef: Error in the matrix inversion used in lemma 2 for the Fourier basis
%               Ef(i,j)=Error(inv(inv(M F')),M F')
%             with M a moment matrix basis of size i,j
%                  F a Fourier matrix basis of size i,j
%       > Eh: Error in the matrix inversion used in lemma 2 for the Haar basis
%               Ef(i,j)=Error(inv(inv(M H')),M H')
%             with M a moment matrix basis of size i,j
%                  H a Haar matrix basis of size i,j
%       > Ec: Error in the matrix inversion used in lemma 2 for the  basis
%               Ef(i,j)=Error(inv(inv(M H')),M H')
%             with M a moment matrix basis of size i,j
%                  C a Chebyshev matrix basis of size i,j
% ---------------------------------------------------------------
function [Em,Ef,Eh,Ec]=Test(ma)
  
  Em=-1*ones(ma,ma);
  Ef=-1*ones(ma,ma);
  Eh=-1*ones(ma,ma);
  Ec=-1*ones(ma,ma);

  for i=1:ma  %image size
    for j=1:i % the moment order is < image size
       M=TranslatedMomentsBasis(i,j);
       
       %Em(i,j)
       O=M'*M; %the original
       iO=InvertMatrix(O); %the inversion
       Em(i,j)=Error(O*iO,eye(j)); %see if it really works as inverse
 
       %Ef(i,j)
       F=FourierBasis(i,j);
       O=M'*F; %the original
       iO=InvertMatrix(O); %the inversion
       Ef(i,j)=Error(real(O*iO),eye(j)); %see if it really works as inverse
       
       %Eh(i,j)
       if (fix(log2(i))==log2(i))
         H=HaarBasis(i,j);
         O=M'*H; %the original
         iO=InvertMatrix(O); %the inversion
         Eh(i,j)=Error(O*iO,eye(j)); %see if it really works as inverse
       end
         
       %Ec(i,j)
       C=ChebyshevBasis(i,j);
       O=M'*C; %the original
       iO=InvertMatrix(O); %the inversion
       Ec(i,j)=Error(O*iO,eye(j)); %see if it really works as inverse
       
    end
  end
